from evaluation.nodesel_estimate import NodeselEstimate
from evaluation.nodesel_policy import NodeselPolicy
from sampling.nodesel_oracle import NodeselRestartdfs


class NodeselPolicyRestartdfs(NodeselPolicy):

    def __init__(self, policy, config_file, seed, device, on_both, prune,
                 prune_on_both, only_prune_steps, verbose=False):
        super().__init__(policy, config_file, seed, device, on_both,
                 prune_on_both, only_prune_steps, verbose)
        self.p = NodeselRestartdfs(prune)

    def on_no_children(self):
        return self.p.nodeselect(self.model)

    def on_negative_depth(self):
        return self.p.nodeselect(self.model)

    def nodecomp(self, node1, node2):
        if self.in_no_children or self.in_negative_depth:
            return self.p.nodecomp(node1, node2)
        else:
            return super().nodecomp(node1, node2)


class NodeselPolicyEstimate(NodeselPolicy):
    def __init__(self, policy, config_file, seed, device, on_both, prune,
                 prune_on_both, only_prune_steps, verbose=False):
        super().__init__(policy, config_file, seed, device, on_both,
                         prune_on_both, only_prune_steps, verbose)
        self.p = NodeselEstimate(prune)

    def on_no_children(self):
        return self.p.nodeselect(self.model)

    def on_negative_depth(self):
        return self.p.nodeselect(self.model)

    def nodecomp(self, node1, node2):
        if self.in_no_children or self.in_negative_depth:
            return self.p.nodecomp(node1, node2)
        else:
            return super().nodecomp(node1, node2)


class NodeselPolicyScore(NodeselPolicy):
    def __init__(self, policy, config_file, seed, device, on_both, prune,
                 prune_on_both, only_prune_steps, verbose=False):
        super().__init__(policy, config_file, seed, device, on_both,
                         prune_on_both, only_prune_steps, verbose)
        self.p = NodeselEstimate(prune)

    def on_no_children(self):
        res = {'selnode': self.model.getBestNode()}
        return res

    def on_negative_depth(self):
        return self.p.nodeselect(self.model)

    def nodecomp(self, node1, node2):
        if self.in_negative_depth:
            return self.p.nodecomp(node1, node2)
        else:
            return super().nodecomp(node1, node2)