class HeFeats:
    def __init__(self, rootlpobj, sumobjcoeff, nconstrs, maxdepth, depth, size, boundtype, he_features):
        self.rootlpobj = rootlpobj
        self.sumobjcoeff = sumobjcoeff
        self.nconstrs = nconstrs
        self.maxdepth = maxdepth
        self.depth = depth
        self.size = size
        self.boundtype = boundtype
        self.vals = he_features

    @staticmethod
    def create(model, size, vals):
        maxdepth = model.getNBinVars() + model.getNIntVars()
        return HeFeats(0, 0, 0, maxdepth, 0, size, 0, vals)

    def get_offset(self):
        return int((self.size * 2) * (self.depth / (self.maxdepth / 10)) + (self.size * self.boundtype))

    def calc(self, model, node):
        self.vals.calc(model, node, self)

class HeFeaturesNodesel:
    size = 18

    def __init__(self, lowerbound, estimate, type_sibling, type_child, type_leaf,
                 branchvar_boundlpdiff, branchvar_rootlpdiff, branchvar_prio_up, branchvar_prio_down, branchvar_pseudocost,
                 branchvar_inf,relativebound, globalupperbound, gap, gapinf,
                 globalupperboundinf, plungedepth, relativedepth):
        self.lowerbound = lowerbound
        self.estimate = estimate
        self.type_sibling = type_sibling
        self.type_child = type_child
        self.type_leaf = type_leaf
        self.branchvar_boundlpdiff = branchvar_boundlpdiff
        self.branchvar_rootlpdiff = branchvar_rootlpdiff
        self.branchvar_prio_up = branchvar_prio_up
        self.branchvar_prio_down = branchvar_prio_down
        self.branchvar_pseudocost = branchvar_pseudocost
        self.branchvar_inf = branchvar_inf
        self.relativebound = relativebound
        self.globalupperbound = globalupperbound
        self.gap = gap
        self.gapinf = gapinf
        self.globalupperbound = globalupperbound
        self.globalupperboundinf = globalupperboundinf
        self.plungedepth = plungedepth
        self.relativedepth = relativedepth

    def __getitem__(self, index):
        d = self.__dict__
        k = list(d.keys())[index]
        return d[k]

    @staticmethod
    def default():
        return HeFeaturesNodesel(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)

    def calc(self, model, node, hefeats):
        assert node.getDepth() != 0
        assert hefeats.maxdepth != 0

        branchinfos = node.getBranchInfos()

        nodetype = node.getType()
        nodelowerbound = node.getLowerbound()
        rootlowerbound = abs(model.getLowerboundRoot())
        if model.isInfinity(rootlowerbound):
            raise Exception('Lowerbound root is infinity')
        if rootlowerbound == 0:
            rootlowerbound = 0.0001

        lowerbound = model.getLowerbound()
        upperbound = model.getUpperbound()

        if model.isInfinity(upperbound) or model.isInfinity(-1*upperbound):
            upperboundinf = True
        else:
            upperboundinf = False

        depth = node.getDepth()
        hefeats.depth = depth

        if upperbound == lowerbound:
            self.gap = 0
        elif lowerbound == 0 or upperboundinf:
            self.gapinf = 1
        else:
            self.gap = (upperbound - lowerbound) / abs(lowerbound)

        if upperboundinf:
            self.globalupperboundinf = 1
            upperbound = lowerbound + 0.2 * (upperbound - lowerbound)
        else:
            self.globalupperbound = upperbound / rootlowerbound

        self.plungedepth = model.getPlungeDepth()
        self.relativedepth = depth / hefeats.maxdepth * 10

        # currently only support branching on one variable

        branchvar = branchinfos[1]
        branchbound = branchinfos[0]
        branchdirpreferred = branchvar.getBranchDirection()

        varsol = branchvar.getLPSol()
        varrootsol = branchvar.getRootSol()

        boundtype = branchinfos[3]
        hefeats.boundtype = boundtype

        # calculate features

        self.lowerbound = nodelowerbound / rootlowerbound
        self.estimate = node.getEstimate() / rootlowerbound

        if lowerbound != upperbound:
            self.relativebound = (nodelowerbound - lowerbound) / (upperbound - lowerbound)

        sibling_type = 2
        child_type = 3
        leaf_type = 4

        if nodetype == sibling_type:
            self.type_sibling = 1
        elif nodetype == child_type:
            self.type_child = 1
        elif nodetype == leaf_type:
            self.type_leaf = 1

        self.branchvar_boundlpdiff = branchbound - varsol
        self.branchvar_rootlpdiff = varrootsol - varsol


        branchdir_downwards = 0
        branchdir_upwards = 1

        if branchdirpreferred == branchdir_downwards:
            self.feat_branchvar_prio_down = 1
        elif branchdirpreferred == branchdir_upwards:
            self.feat_branchvar_prio_up = 1

        self.branchvar_pseudocost = model.varGetPseudocost(branchvar, branchbound - varsol)

        boundtype_lower = 0
        if boundtype == boundtype_lower:
            self.branchvar_inf = model.varGetAvgInferences(branchvar, branchdir_upwards) / hefeats.maxdepth
        else:
            self.branchvar_inf = model.varGetAvgInferences(branchvar, branchdir_downwards) / hefeats.maxdepth

        hefeats.vals = self


class HeFeaturesNodepru:
    size = 16

    def __init__(self, globallowerbound, globalupperbound, gap, nsolution, plungedepth,
                 relativedepth, relativebound, relativeestimate, gapinf, globalupperboundinf,
                 branchvar_boundlpdiff, branchvar_rootlpdiff, branchvar_prio_up, branchvar_prio_down,
                 branchvar_pseudocost, branchvar_inf):
        self.globallowerbound = globallowerbound
        self.globalupperbound = globalupperbound
        self.gap = gap
        self.nsolution = nsolution
        self.plungedepth = plungedepth
        self.relativedepth = relativedepth
        self.relativebound = relativebound
        self.relativeestimate = relativeestimate
        self.gapinf = gapinf
        self.globalupperboundinf = globalupperboundinf
        self.branchvar_boundlpdiff = branchvar_boundlpdiff
        self.branchvar_rootlpdiff = branchvar_rootlpdiff
        self.branchvar_prio_up = branchvar_prio_up
        self.branchvar_prio_down = branchvar_prio_down
        self.branchvar_pseudocost = branchvar_pseudocost
        self.branchvar_inf = branchvar_inf

    def __getitem__(self, index):
        d = self.__dict__
        k = list(d.keys())[index]
        return d[k]

    @staticmethod
    def default():
        return HeFeaturesNodepru(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)

    def calc(self, model, node, hefeats):
        assert node.getDepth() != 0
        assert hefeats.maxdepth != 0

        branchinfos = node.getBranchInfos()

        depth = node.getDepth()
        hefeats.depth = depth

        lowerbound = model.getLowerbound()
        assert not model.isInfinity(lowerbound)

        upperbound = model.getUpperbound()
        if model.isInfinity(upperbound) or model.isInfinity(-1*upperbound):
            upperboundinf = True
        else:
            upperboundinf = False

        rootlowerbound = abs(model.getLowerboundRoot())
        if model.isInfinity(rootlowerbound):
            raise Exception('Lowerbound root is infinity')
        if rootlowerbound == 0:
            rootlowerbound = 0.0001

        # currently only support branching on one variable

        branchvar = branchinfos[1]
        branchbound = branchinfos[0]
        branchdirpreferred = branchvar.getBranchDirection()

        varsol = branchvar.getLPSol()
        varrootsol = branchvar.getRootSol()

        boundtype = branchinfos[3]
        hefeats.boundtype = boundtype

        # calculate features
        if upperbound == lowerbound:
            self.gap = 0
        elif lowerbound == 0 or upperboundinf:
            self.gapinf = 1
        else:
            self.gap = (upperbound - lowerbound) / abs(lowerbound)

        self.globallowerbound = lowerbound / rootlowerbound
        if upperboundinf:
            self.globalupperboundinf = 1
        else:
            self.globalupperbound = upperbound / rootlowerbound

        self.nsolution = model.getNSolsFound()
        self.plungedepth = model.getPlungeDepth()
        self.relativedepth = depth / hefeats.maxdepth * 10

        if upperboundinf:
            upperbound = lowerbound + 0.2 * (upperbound - lowerbound)

        if upperbound != lowerbound:
            self.relativebound = (node.getLowerbound() - lowerbound) / (upperbound - lowerbound)
            self.relativeestimate = (node.getEstimate() - lowerbound) / (upperbound - lowerbound)

        self.branchvar_boundlpdiff = branchbound - varsol
        self.branchvar_rootlpdiff = varrootsol - varsol

        branchdir_downwards = 0
        branchdir_upwards = 1

        if branchdirpreferred == branchdir_downwards:
            self.feat_branchvar_prio_down = 1
        elif branchdirpreferred == branchdir_upwards:
            self.feat_branchvar_prio_up = 1

        self.branchvar_pseudocost = model.varGetPseudocost(branchvar, branchbound - varsol)

        boundtype_lower = 0
        if boundtype == boundtype_lower:
            self.branchvar_inf = model.varGetAvgInferences(branchvar, branchdir_upwards) / hefeats.maxdepth
        else:
            self.branchvar_inf = model.varGetAvgInferences(branchvar, branchdir_downwards) / hefeats.maxdepth

        hefeats.vals = self
