import pyscipopt as scip

from he.he_feats import HeFeats, HeFeaturesNodepru


class NodepruHePolicy(scip.Nodepru):
    def __init__(self, policy):
        super().__init__()
        self.policy = policy
        self.number_nodes_pruned = 0
        self.iters = 0

    def nodeinit(self):
        self.he_feats = HeFeats.create(self.model, HeFeaturesNodepru.size, HeFeaturesNodepru.default())

    def nodeprune(self):
        node = self.model.getCurrentNode()
        self.iters += 1
        # don't prune the root
        if node.getDepth() == 0:
            return {'result': False}
        else:
            self.he_feats.calc(self.model, node)
            score = self.policy.calc_node_score(self.he_feats)
            if score > 0:
                self.number_nodes_pruned += 1
                return {'result': True}
            else:
                return {'result': False}
