import pyscipopt as scip

from he.he_feats import HeFeats, HeFeaturesNodesel


class NodeselHePolicy(scip.Nodesel):
    def __init__(self, policy, stop_first_leaf):
        super().__init__()
        self.policy = policy
        self.node_to_score = {}
        self.stop_first_leaf = stop_first_leaf

    def nodeinit(self):
        self.he_feats = HeFeats.create(self.model, HeFeaturesNodesel.size, HeFeaturesNodesel.default())

    def nodeselect(self):
        children = self.model.getChildren()
        if self.model.getDepth() > 0 and len(children) == 0 and self.stop_first_leaf:
            self.model.interruptSolve()
        for child in children:
            self.he_feats.calc(self.model, child)
            self.node_to_score[child.getNumber()] = self.policy.calc_node_score(self.he_feats)
        return {"selnode": self.model.getBestNode()}

    def nodecomp(self, node1, node2):
        score1 = self.node_to_score[node1.getNumber()]
        score2 = self.node_to_score[node2.getNumber()]
        if score1 > score2:
            return -1
        elif score1 < score2:
            return 1
        else:
            depth1 = node1.getDepth()
            depth2 = node2.getDepth()
            if depth1 > depth2:
                return -1
            elif depth1 < depth2:
                return 1
            else:
                lowerbound1 = node1.getLowerbound()
                lowerbound2 = node2.getLowerbound()
                if lowerbound1 > lowerbound2:
                    return 1
                elif lowerbound1 < lowerbound2:
                    return -1
                else:
                    return 0