from pyscipopt import scip
import utilities


class Policy:
    def __init__(self, weights):
        self.weights = weights

    @staticmethod
    def from_file(path):
        with open(path, 'r') as f:
            lines = f.readlines()
        header_size = 6
        return Policy([float(x) for x in lines[header_size:]])

    def calc_node_score(self, he_feats):
        offset = he_feats.get_offset()
        score = 0
        weights = self.weights
        featvals = he_feats.vals

        if offset + he_feats.size <= len(weights):
            for i in range(he_feats.size):
                score += featvals[i] * weights[i+offset]
        return score


def init_scip_model(instance):
    m = scip.Model()
    m.setIntParam('display/verblevel', 0)
    m.readProblem(f'{instance}')
    utilities.init_scip_params(m, seed=0, heuristics=False, presolving=False)
    m.setIntParam('timing/clocktype', 2)
    return m
