from torch.utils.data import Dataset as AbstractDataset
import torch
import pandas as pd


class Dataset(AbstractDataset):

    def __init__(self, list_of_filenames, device):
        super().__init__()
        if not isinstance(list_of_filenames, list):
            self.df = pd.read_hdf(list_of_filenames)
            self.no_sequence = True
        else:
            self.data_files = list_of_filenames
            self.no_sequence = False
        self.device = device

    def __getitem__(self, item):
        if self.no_sequence:
            if not isinstance(self.df, pd.DataFrame):
                return
            return torch.tensor(self.df.iloc[item].values, dtype=torch.float32)
        else:
            return self.load_file(self.data_files[item])

    def __len__(self):
        if self.no_sequence and isinstance(self.df, pd.DataFrame):
            return self.df.shape[0]
        else:
            return len(self.data_files)

    def load_file(self, filename):
        file = pd.read_hdf(filename)
        arr = file.values
        tensor = torch.tensor(arr, dtype=torch.float32)
        return tensor

