from collections import OrderedDict

import torch.nn as nn


class NeuralNet(nn.Module):

    def __init__(self, n_feats, n_hidden_layers, hidden_units, dropout_rate, k):
        super().__init__()
        n_output = 1 if k == 1 else 3
        if n_hidden_layers == 0:
            self.nn = nn.Sequential(nn.Linear(n_feats, n_output))
        else:
            layers = [
                ('input', nn.Linear(n_feats, hidden_units)),
                (f'relu_input', nn.ReLU()),
                (f'batch_norm', nn.BatchNorm1d(hidden_units)),
                (f'dropout', nn.Dropout(dropout_rate))
            ]
            for i in range(n_hidden_layers):
                layers.append((f'linear_{i}', nn.Linear(hidden_units, hidden_units)))
                layers.append((f'relu_{i}', nn.ReLU()))
                layers.append((f'batch_norm_{i}', nn.BatchNorm1d(hidden_units)))
                layers.append((f'dropout_{i}', nn.Dropout(dropout_rate)))
            layers.append(('output', nn.Linear(hidden_units, n_output)))
            self.nn = nn.Sequential(
                OrderedDict(layers)
            )

    def forward(self, inputs):
        return self.nn(inputs)

    def to(self, *args, **kwargs):
        self = super().to(*args, **kwargs)
        self.nn = self.nn.to(*args, **kwargs)
        return self