import pickle
import math
import time


def one_sample_dict(obj, config=None, config_file_path=None):
    if config is None and config_file_path is not None:
        with open(config_file_path, 'rb') as f:
            config = pickle.load(f)

    start_time = time.time()
    for k in config['removed_columns']:
        del obj[k]
    dropping_cols = time.time() - start_time

    start_time = time.time()
    obj['inc_val'] = -1 if math.isnan(obj['inc_val']) else obj['inc_val']
    obj['avg_inc_val'] = -1 if math.isnan(obj['avg_inc_val']) else obj['avg_inc_val']
    removing_nan = time.time() - start_time

    transforming = 0  # not done here

    start_time = time.time()
    cols = config['standardized'].keys()
    for column in cols:
        mu = config['standardized'][column]['mu']
        std = config['standardized'][column]['std']
        obj[column] = (obj[column] - mu) / std
    standardizing = time.time() - start_time

    return config, dropping_cols, removing_nan, transforming, standardizing

def one_sample(df, config=None, config_file_path=None):
    if config is None and config_file_path is not None:
        with open(config_file_path, 'rb') as f:
            config = pickle.load(f)

    start_time = time.time()
    df.drop(config['removed_columns'], inplace=True)
    dropping_cols = time.time() - start_time

    start_time = time.time()
    df['inc_val'] = -1 if math.isnan(df['inc_val']) else df['inc_val']
    df['avg_inc_val'] = -1 if math.isnan(df['avg_inc_val']) else df['avg_inc_val']
    removing_nan = time.time() - start_time

    start_time = time.time()
    transforming = time.time() - start_time

    start_time = time.time()
    cols = config['standardized'].keys()
    for column in cols:
        mu = config['standardized'][column]['mu']
        std = config['standardized'][column]['std']
        df[column] = (df[column] - mu) / std
    standardizing = time.time() - start_time

    return config, dropping_cols, removing_nan, transforming, standardizing
